@@ -17,6 +17,7 @@
 
 import errno
 import os
+import operator
 import sys
 import hashlib
 import yaml
@@ -27,12 +28,14 @@ from xml.dom import minidom
 import jenkins
 import re
 import pkg_resources
+from pprint import pformat
 import logging
 import copy
 import itertools
 import fnmatch
+import six
 from jenkins_jobs.errors import JenkinsJobsException
-import local_yaml
+import jenkins_jobs.local_yaml as local_yaml
 
 logger = logging.getLogger(__name__)
 MAGIC_MANAGE_STRING = "<!-- Managed by Jenkins Job Builder -->"
@@ -40,7 +43,7 @@ MAGIC_MANAGE_STRING = "<!-- Managed by Jenkins Job Builder -->"
 
 # Python 2.6's minidom toprettyxml produces broken output by adding extraneous
 # whitespace around data. This patches the broken implementation with one taken
-# from 2.7
+# from Python > 2.7.3
 def writexml(self, writer, indent="", addindent="", newl=""):
     # indent = current indentation
     # addindent = indentation to add to higher levels
@@ -71,7 +74,7 @@ def writexml(self, writer, indent="", addindent="", newl=""):
 
 # PyXML xml.__name__ is _xmlplus. Check that if we don't have the default
 # system version of the minidom, then patch the writexml method
-if sys.version_info[:3] < (2, 7, 0) or xml.__name__ != 'xml':
+if sys.version_info[:3] < (2, 7, 3) or xml.__name__ != 'xml':
     minidom.Element.writexml = writexml
 
 
@@ -82,7 +85,7 @@ def deep_format(obj, paramdict):
     # limitations on the values in paramdict - the post-format result must
     # still be valid YAML (so substituting-in a string containing quotes, for
     # example, is problematic).
-    if isinstance(obj, basestring):
+    if hasattr(obj, 'format'):
         try:
             result = re.match('^{obj:(?P<key>\w+)}$', obj)
             if result is not None:
@@ -91,8 +94,8 @@ def deep_format(obj, paramdict):
                 ret = obj.format(**paramdict)
         except KeyError as exc:
             missing_key = exc.message
-            desc = "%s parameter missing to format %s\nGiven: %s" % (
-                   missing_key, obj, paramdict)
+            desc = "%s parameter missing to format %s\nGiven:\n%s" % (
+                   missing_key, obj, pformat(paramdict))
             raise JenkinsJobsException(desc)
     elif isinstance(obj, list):
         ret = []
@@ -101,7 +104,14 @@ def deep_format(obj, paramdict):
     elif isinstance(obj, dict):
         ret = {}
         for item in obj:
-            ret[item.format(**paramdict)] = deep_format(obj[item], paramdict)
+            try:
+                ret[item.format(**paramdict)] = \
+                    deep_format(obj[item], paramdict)
+            except KeyError as exc:
+                missing_key = exc.message
+                desc = "%s parameter missing to format %s\nGiven:\n%s" % (
+                    missing_key, obj, pformat(paramdict))
+                raise JenkinsJobsException(desc)
     else:
         ret = obj
     return ret
@@ -124,6 +134,7 @@ class YamlParser(object):
     def __init__(self, config=None):
         self.data = {}
         self.jobs = []
+        self.xml_jobs = []
         self.config = config
         self.registry = ModuleRegistry(self.config)
         self.path = ["."]
@@ -132,6 +143,15 @@ class YamlParser(object):
                     config.has_option('job_builder', 'include_path'):
                 self.path = config.get('job_builder',
                                        'include_path').split(':')
+        self.keep_desc = self.get_keep_desc()
+
+    def get_keep_desc(self):
+        keep_desc = False
+        if self.config and self.config.has_section('job_builder') and \
+                self.config.has_option('job_builder', 'keep_descriptions'):
+            keep_desc = self.config.getboolean('job_builder',
+                                               'keep_descriptions')
+        return keep_desc
 
     def parse_fp(self, fp):
         data = local_yaml.load(fp, search_path=self.path)
@@ -142,7 +162,7 @@ class YamlParser(object):
                     " not a {cls}".format(fname=getattr(fp, 'name', fp),
                                           cls=type(data)))
             for item in data:
-                cls, dfn = item.items()[0]
+                cls, dfn = next(iter(item.items()))
                 group = self.data.get(cls, {})
                 if len(item.items()) > 1:
                     n = None
@@ -155,6 +175,9 @@ class YamlParser(object):
                                                "named '{0}'. Missing indent?"
                                                .format(n))
                 name = dfn['name']
+                if name in group:
+                    self._handle_dups("Duplicate entry found: '{0}' is "
+                                      "already defined".format(name))
                 group[name] = dfn
                 self.data[cls] = group
 
@@ -162,6 +185,15 @@ class YamlParser(object):
         with open(fn) as fp:
             self.parse_fp(fp)
 
+    def _handle_dups(self, message):
+
+        if not (self.config and self.config.has_section('job_builder') and
+                self.config.getboolean('job_builder', 'allow_duplicates')):
+            logger.error(message)
+            raise JenkinsJobsException(message)
+        else:
+            logger.warn(message)
+
     def getJob(self, name):
         job = self.data.get('job', {}).get(name, None)
         if not job:
@@ -188,7 +220,16 @@ class YamlParser(object):
         newdata.update(data)
         return newdata
 
-    def generateXML(self, jobs_filter=None):
+    def formatDescription(self, job):
+        if self.keep_desc:
+            description = job.get("description", None)
+        else:
+            description = job.get("description", '')
+        if description is not None:
+            job["description"] = description + \
+                self.get_managed_string().lstrip()
+
+    def expandYaml(self, jobs_filter=None):
         changed = True
         while changed:
             changed = False
@@ -201,15 +242,18 @@ class YamlParser(object):
             if jobs_filter and not matches(job['name'], jobs_filter):
                 logger.debug("Ignoring job {0}".format(job['name']))
                 continue
-            logger.debug("XMLifying job '{0}'".format(job['name']))
+            logger.debug("Expanding job '{0}'".format(job['name']))
             job = self.applyDefaults(job)
-            self.getXMLForJob(job)
+            self.formatDescription(job)
+            self.jobs.append(job)
         for project in self.data.get('project', {}).values():
-            logger.debug("XMLifying project '{0}'".format(project['name']))
+            logger.debug("Expanding project '{0}'".format(project['name']))
+            # use a set to check for duplicate job references in projects
+            seen = set()
             for jobspec in project.get('jobs', []):
                 if isinstance(jobspec, dict):
                     # Singleton dict containing dict of job-specific params
-                    jobname, jobparams = jobspec.items()[0]
+                    jobname, jobparams = next(iter(jobspec.items()))
                     if not isinstance(jobparams, dict):
                         jobparams = {}
                 else:
@@ -218,6 +262,11 @@ class YamlParser(object):
                 job = self.getJob(jobname)
                 if job:
                     # Just naming an existing defined job
+                    if jobname in seen:
+                        self._handle_dups("Duplicate job '{0}' specified "
+                                          "for project '{1}'".format(
+                                              jobname, project['name']))
+                    seen.add(jobname)
                     continue
                 # see if it's a job group
                 group = self.getJobGroup(jobname)
@@ -225,7 +274,7 @@ class YamlParser(object):
                     for group_jobspec in group['jobs']:
                         if isinstance(group_jobspec, dict):
                             group_jobname, group_jobparams = \
-                                group_jobspec.items()[0]
+                                next(iter(group_jobspec.items()))
                             if not isinstance(group_jobparams, dict):
                                 group_jobparams = {}
                         else:
@@ -233,6 +282,12 @@ class YamlParser(object):
                             group_jobparams = {}
                         job = self.getJob(group_jobname)
                         if job:
+                            if group_jobname in seen:
+                                self._handle_dups(
+                                    "Duplicate job '{0}' specified for "
+                                    "project '{1}'".format(group_jobname,
+                                                           project['name']))
+                            seen.add(group_jobname)
                             continue
                         template = self.getJobTemplate(group_jobname)
                         # Allow a group to override parameters set by a project
@@ -244,7 +299,8 @@ class YamlParser(object):
                         # Except name, since the group's name is not useful
                         d['name'] = project['name']
                         if template:
-                            self.getXMLForTemplateJob(d, template, jobs_filter)
+                            self.expandYamlForTemplateJob(d, template,
+                                                          jobs_filter)
                     continue
                 # see if it's a template
                 template = self.getJobTemplate(jobname)
@@ -252,13 +308,22 @@ class YamlParser(object):
                     d = {}
                     d.update(project)
                     d.update(jobparams)
-                    self.getXMLForTemplateJob(d, template, jobs_filter)
+                    self.expandYamlForTemplateJob(d, template, jobs_filter)
                 else:
                     raise JenkinsJobsException("Failed to find suitable "
                                                "template named '{0}'"
                                                .format(jobname))
+        # check for duplicate generated jobs
+        seen = set()
+        # walk the list in reverse so that last definition wins
+        for job in self.jobs[::-1]:
+            if job['name'] in seen:
+                self._handle_dups("Duplicate definitions for job '{0}' "
+                                  "specified".format(job['name']))
+                self.jobs.remove(job)
+            seen.add(job['name'])
 
-    def getXMLForTemplateJob(self, project, template, jobs_filter=None):
+    def expandYamlForTemplateJob(self, project, template, jobs_filter=None):
         dimensions = []
         for (k, v) in project.items():
             if type(v) == list and k not in ['jobs']:
@@ -275,7 +340,7 @@ class YamlParser(object):
             expanded_values = {}
             for (k, v) in values:
                 if isinstance(v, dict):
-                    inner_key = v.iterkeys().next()
+                    inner_key = next(iter(v))
                     expanded_values[k] = inner_key
                     expanded_values.update(v[inner_key])
                 else:
@@ -295,40 +360,34 @@ class YamlParser(object):
             # us guarantee a group of parameters will not be added a
             # second time.
             uniq = json.dumps(expanded, sort_keys=True)
+            if six.PY3:
+                uniq = uniq.encode('utf-8')
             checksum = hashlib.md5(uniq).hexdigest()
 
             # Lookup the checksum
             if checksum not in checksums:
-
-                # We also want to skip XML generation whenever the user did
+                # We also want to skip expansion whenever the user did
                 # not ask for that job.
                 job_name = expanded.get('name')
                 if jobs_filter and not matches(job_name, jobs_filter):
                     continue
 
-                logger.debug("Generating XML for template job {0}"
-                             " (params {1})".format(
-                                 template['name'], params))
-                self.getXMLForJob(expanded)
+                self.formatDescription(expanded)
+                self.jobs.append(expanded)
                 checksums.add(checksum)
 
+    def get_managed_string(self):
+        # The \n\n is not hard coded, because they get stripped if the
+        # project does not otherwise have a description.
+        return "\n\n" + MAGIC_MANAGE_STRING
+
+    def generateXML(self):
+        for job in self.jobs:
+            self.xml_jobs.append(self.getXMLForJob(job))
+
     def getXMLForJob(self, data):
         kind = data.get('project-type', 'freestyle')
 
-        keep_desc = False
-        if self.config:
-            if self.config.has_section('job_builder') and \
-                    self.config.has_option('job_builder', 'keep_descriptions'):
-                keep_desc = self.config.getboolean('job_builder',
-                                                   'keep_descriptions')
-
-        if keep_desc:
-            description = data.get("description", None)
-        else:
-            description = data.get("description", '')
-        if description is not None:
-            data["description"] = description + \
-                self.get_managed_string().lstrip()
         for ep in pkg_resources.iter_entry_points(
                 group='jenkins_jobs.projects', name=kind):
             Mod = ep.load()
@@ -336,19 +395,13 @@ class YamlParser(object):
             xml = mod.root_xml(data)
             self.gen_xml(xml, data)
             job = XmlJob(xml, data['name'])
-            self.jobs.append(job)
-            break
+            return job
 
     def gen_xml(self, xml, data):
         for module in self.registry.modules:
             if hasattr(module, 'gen_xml'):
                 module.gen_xml(self, xml, data)
 
-    def get_managed_string(self):
-        # The \n\n is not hard coded, because they get stripped if the
-        # project does not otherwise have a description.
-        return "\n\n" + MAGIC_MANAGE_STRING
-
 
 class ModuleRegistry(object):
     entry_points_cache = {}
@@ -364,7 +417,7 @@ class ModuleRegistry(object):
             Mod = entrypoint.load()
             mod = Mod(self)
             self.modules.append(mod)
-            self.modules.sort(lambda a, b: cmp(a.sequence, b.sequence))
+            self.modules.sort(key=operator.attrgetter('sequence'))
             if mod.component_type is not None:
                 self.modules_by_component_type[mod.component_type] = mod
 
@@ -408,7 +461,7 @@ class ModuleRegistry(object):
 
         if isinstance(component, dict):
             # The component is a singleton dictionary of name: dict(args)
-            name, component_data = component.items()[0]
+            name, component_data = next(iter(component.items()))
             if template_data:
                 # Template data contains values that should be interpolated
                 # into the component definition
@@ -609,12 +662,15 @@ class Builder(object):
     def delete_job(self, glob_name, fn=None):
         if fn:
             self.load_files(fn)
-            self.parser.generateXML(glob_name)
-            jobs = [j.name
+            self.parser.expandYaml(glob_name)
+            jobs = [j['name']
                     for j in self.parser.jobs
-                    if matches(j.name, [glob_name])]
+                    if matches(j['name'], [glob_name])]
         else:
             jobs = [glob_name]
+
+        if jobs is not None:
+            logger.info("Removing jenkins job(s): %s" % ", ".join(jobs))
         for job in jobs:
             self.jenkins.delete_job(job)
             if(self.cache.is_cached(job)):
@@ -627,11 +683,12 @@ class Builder(object):
 
     def update_job(self, input_fn, names=None, output=None):
         self.load_files(input_fn)
-        self.parser.generateXML(names)
+        self.parser.expandYaml(names)
+        self.parser.generateXML()
 
-        self.parser.jobs.sort(lambda a, b: cmp(a.name, b.name))
+        self.parser.xml_jobs.sort(key=operator.attrgetter('name'))
 
-        for job in self.parser.jobs:
+        for job in self.parser.xml_jobs:
             if names and not matches(job.name, names):
                 continue
             if output:
@@ -674,4 +731,4 @@ class Builder(object):
                 self.cache.set(job.name, md5)
             else:
                 logger.debug("'{0}' has not changed".format(job.name))
-        return self.parser.jobs
+        return self.parser.xml_jobs
