@@ -14,12 +14,12 @@
 # under the License.
 
 import argparse
-import ConfigParser
+from six.moves import configparser, StringIO
 import logging
 import os
 import platform
 import sys
-import cStringIO
+import jenkins_jobs.version
 
 from jenkins_jobs.builder import Builder
 from jenkins_jobs.errors import JenkinsJobsException
@@ -32,11 +32,15 @@ DEFAULT_CONF = """
 keep_descriptions=False
 ignore_cache=False
 recursive=False
+allow_duplicates=False
 
 [jenkins]
 url=http://localhost:8080/
 user=
 password=
+
+[hipchat]
+authtoken=dummy
 """
 
 
@@ -66,13 +70,15 @@ def create_parser():
     subparser = parser.add_subparsers(help='update, test or delete job',
                                       dest='command')
     parser_update = subparser.add_parser('update', parents=[recursive_parser])
-    parser_update.add_argument('path', help='path to YAML file or directory')
+    parser_update.add_argument('path', help='colon-separated list of paths to'
+                                            ' YAML files or directories')
     parser_update.add_argument('names', help='name(s) of job(s)', nargs='*')
     parser_update.add_argument('--delete-old', help='delete obsolete jobs',
                                action='store_true',
                                dest='delete_old', default=False,)
     parser_test = subparser.add_parser('test', parents=[recursive_parser])
-    parser_test.add_argument('path', help='path to YAML file or directory',
+    parser_test.add_argument('path', help='colon-separated list of paths to'
+                                          ' YAML files or directories',
                              nargs='?', default=sys.stdin)
     parser_test.add_argument('-o', dest='output_dir', default=sys.stdout,
                              help='path to output XML')
@@ -80,7 +86,8 @@ def create_parser():
     parser_delete = subparser.add_parser('delete')
     parser_delete.add_argument('name', help='name of job', nargs='+')
     parser_delete.add_argument('-p', '--path', default=None,
-                               help='path to YAML file or directory')
+                               help='colon-separated list of paths to'
+                                    ' YAML files or directories')
     subparser.add_parser('delete-all',
                          help='delete *ALL* jobs from Jenkins server, '
                          'including those not managed by Jenkins Job '
@@ -96,6 +103,9 @@ def create_parser():
     parser.add_argument(
         '--flush-cache', action='store_true', dest='flush_cache',
         default=False, help='flush all the cache entries before updating')
+    parser.add_argument('--version', dest='version', action='version',
+                        version=version(),
+                        help='show version')
 
     return parser
 
@@ -110,6 +120,8 @@ def main(argv=None):
 
     parser = create_parser()
     options = parser.parse_args(argv)
+    if not options.command:
+        parser.error("Must specify a 'command' to be performed")
     if (options.log_level is not None):
         options.log_level = getattr(logging, options.log_level.upper(),
                                     logger.getEffectiveLevel())
@@ -130,9 +142,9 @@ def setup_config_settings(options):
                                  'jenkins_jobs.ini')
         if os.path.isfile(localconf):
             conf = localconf
-    config = ConfigParser.ConfigParser()
+    config = configparser.ConfigParser()
     ## Load default config always
-    config.readfp(cStringIO.StringIO(DEFAULT_CONF))
+    config.readfp(StringIO(DEFAULT_CONF))
     if os.path.isfile(conf):
         logger.debug("Reading config from {0}".format(conf))
         conffp = open(conf, 'r')
@@ -167,11 +179,11 @@ def execute(options, config):
     # https://bugs.launchpad.net/openstack-ci/+bug/1259631
     try:
         user = config.get('jenkins', 'user')
-    except (TypeError, ConfigParser.NoOptionError):
+    except (TypeError, configparser.NoOptionError):
         user = None
     try:
         password = config.get('jenkins', 'password')
-    except (TypeError, ConfigParser.NoOptionError):
+    except (TypeError, configparser.NoOptionError):
         password = None
 
     builder = Builder(config.get('jenkins', 'url'),
@@ -181,7 +193,7 @@ def execute(options, config):
                       ignore_cache=ignore_cache,
                       flush_cache=options.flush_cache)
 
-    if hasattr(options, 'path'):
+    if getattr(options, 'path', None):
         if options.path == sys.stdin:
             logger.debug("Input file is stdin")
             if options.path.isatty():
@@ -190,17 +202,22 @@ def execute(options, config):
                     "Reading configuration from STDIN. Press %s to end input.",
                     key)
 
-        # expand or convert options.path to a list
-        if (getattr(options, 'recursive', False)
-            or config.getboolean('job_builder', 'recursive')) and \
-                os.path.isdir(options.path):
-            options.path = recurse_path(options.path)
-        else:
-            options.path = [options.path]
+        # take list of paths
+        options.path = options.path.split(os.pathsep)
+
+        do_recurse = (getattr(options, 'recursive', False) or
+                      config.getboolean('job_builder', 'recursive'))
+
+        paths = []
+        for path in options.path:
+            if do_recurse and os.path.isdir(path):
+                paths.extend(recurse_path(path))
+            else:
+                paths.append(path)
+        options.path = paths
 
     if options.command == 'delete':
         for job in options.name:
-            logger.info("Deleting jobs in [{0}]".format(job))
             builder.delete_job(job, options.path)
     elif options.command == 'delete-all':
         confirm('Sure you want to delete *ALL* jobs from Jenkins server?\n'
@@ -217,6 +234,12 @@ def execute(options, config):
         builder.update_job(options.path, options.name,
                            output=options.output_dir)
 
+
+def version():
+    return "Jenkins Job Builder version: %s" % \
+        jenkins_jobs.version.version_info.version_string()
+
+
 if __name__ == '__main__':
     sys.path.insert(0, '.')
     main()
