@@ -251,7 +251,7 @@ def ant(parser, xml_parent, data):
     if type(data) is str:
         # Support for short form: -ant: "target"
         data = {'targets': data}
-    for setting, value in sorted(data.iteritems()):
+    for setting, value in sorted(data.items()):
         if setting == 'targets':
             targets = XML.SubElement(ant, 'targets')
             targets.text = value
@@ -508,6 +508,8 @@ def inject(parser, xml_parent, data):
 
     :arg str properties-file: the name of the property file (optional)
     :arg str properties-content: the properties content (optional)
+    :arg str script-file: the name of a script file to run (optional)
+    :arg str script-content: the script content (optional)
 
     Example:
 
@@ -520,6 +522,10 @@ def inject(parser, xml_parent, data):
         info, 'propertiesFilePath', data.get('properties-file'))
     jenkins_jobs.modules.base.add_nonblank_xml_subelement(
         info, 'propertiesContent', data.get('properties-content'))
+    jenkins_jobs.modules.base.add_nonblank_xml_subelement(
+        info, 'scriptFilePath', data.get('script-file'))
+    jenkins_jobs.modules.base.add_nonblank_xml_subelement(
+        info, 'scriptContent', data.get('script-content'))
 
 
 def artifact_resolver(parser, xml_parent, data):
@@ -816,6 +822,12 @@ def conditional_step(parser, xml_parent, data):
                        representation of true
 
                          :condition-expression: Expression to expand
+    strings-match      Run the step if two strings match
+
+                         :condition-string1: First string
+                         :condition-string2: Second string
+                         :condition-case-insensitive: Case insensitive
+                           defaults to false
     current-status     Run the build step if the current build status is
                        within the configured range
 
@@ -860,6 +872,14 @@ def conditional_step(parser, xml_parent, data):
                      'org.jenkins_ci.plugins.run_condition.core.'
                      'BooleanCondition')
             XML.SubElement(ctag, "token").text = cdata['condition-expression']
+        elif kind == "strings-match":
+            ctag.set('class',
+                     'org.jenkins_ci.plugins.run_condition.core.'
+                     'StringsMatchCondition')
+            XML.SubElement(ctag, "arg1").text = cdata['condition-string1']
+            XML.SubElement(ctag, "arg2").text = cdata['condition-string2']
+            XML.SubElement(ctag, "ignoreCase").text = str(cdata.get(
+                'condition-case-insensitive', False)).lower()
         elif kind == "current-status":
             ctag.set('class',
                      'org.jenkins_ci.plugins.run_condition.core.'
