@@ -41,6 +41,8 @@ import re
 def gerrit_handle_legacy_configuration(data):
     hyphenizer = re.compile("[A-Z]")
 
+    logger = logging.getLogger("%s:gerrit" % __name__)
+
     def hyphenize(attr):
         """Convert strings like triggerOn to trigger-on.
         """
@@ -50,7 +52,10 @@ def gerrit_handle_legacy_configuration(data):
     def convert_dict(d, old_keys):
         for old_key in old_keys:
             if old_key in d:
-                d[hyphenize(old_key)] = d[old_key]
+                new_key = hyphenize(old_key)
+                logger.warn("'%s' is deprecated and will be removed after "
+                            "1.0.0, please use '%s' instead", old_key, new_key)
+                d[new_key] = d[old_key]
                 del d[old_key]
 
     convert_dict(data, [
@@ -91,7 +96,7 @@ def build_gerrit_triggers(xml_parent, data):
         'hudsontrigger.events'
 
     trigger_on_events = XML.SubElement(xml_parent, 'triggerOnEvents')
-    for config_key, tag_name in available_simple_triggers.iteritems():
+    for config_key, tag_name in available_simple_triggers.items():
         if data.get(config_key, False):
             XML.SubElement(trigger_on_events,
                            '%s.%s' % (tag_namespace, tag_name))
@@ -149,6 +154,7 @@ def gerrit(parser, xml_parent, data):
     :arg str failure-message: Message to leave on failure (default '')
     :arg str successful-message: Message to leave on success (default '')
     :arg str unstable-message: Message to leave when unstable (default '')
+    :arg str notbuilt-message: Message to leave when not built (default '')
     :arg list projects: list of projects to match
 
       :Project: * **project-compare-type** (`str`) --  ''PLAIN'', ''ANT'' or
@@ -221,7 +227,8 @@ def gerrit(parser, xml_parent, data):
 
     Until version 0.4.0 of Jenkins Job Builder, camelCase keys were used to
     configure Gerrit Trigger Plugin, instead of hyphenated-keys.  While still
-    supported, camedCase keys are deprecated and should not be used.
+    supported, camedCase keys are deprecated and should not be used. Support
+    for this will be removed after 1.0.0 is released.
 
     Example:
 
@@ -331,6 +338,8 @@ def gerrit(parser, xml_parent, data):
         data.get('successful-message', ''))
     XML.SubElement(gtrig, 'buildUnstableMessage').text = str(
         data.get('unstable-message', ''))
+    XML.SubElement(gtrig, 'buildNotBuiltMessage').text = str(
+        data.get('notbuilt-message', ''))
     XML.SubElement(gtrig, 'customUrl').text = str(data.get('custom-url', ''))
     XML.SubElement(gtrig, 'serverName').text = str(
         data.get('server-name', '__ANY__'))
@@ -453,7 +462,7 @@ def pollurl(parser, xml_parent, data):
             str(bool(check_content)).lower()
         content_types = XML.SubElement(entry, 'contentTypes')
         for entry in check_content:
-            type_name = entry.keys()[0]
+            type_name = next(iter(entry.keys()))
             if type_name not in valid_content_types:
                 raise JenkinsJobsException('check-content must be one of : %s'
                                            % ', '.join(valid_content_types.
