@@ -57,6 +57,48 @@ def ci_skip(parser, xml_parent, data):
     })
 
 
+def config_file_provider(parser, xml_parent, data):
+    """yaml: config-file-provider
+    Provide configuration files (i.e., settings.xml for maven etc.)
+    which will be copied to the job's workspace.
+    Requires the Jenkins `Config File Provider Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/Config+File+Provider+Plugin>`_
+
+    :arg list files: List of managed config files made up of three\
+      parameters
+
+        :Parameter: * **file-id** (`str`)\
+        The identifier for the managed config file
+        :Parameter: * **target** (`str`)\
+        Define where the file should be created (optional)
+        :Parameter: * **variable** (`str`)\
+        Define an environment variable to be used (optional)
+
+    Example:
+
+    .. literalinclude:: \
+    /../../tests/wrappers/fixtures/config-file-provider003.yaml
+       :language: yaml
+    """
+    top = XML.SubElement(xml_parent, 'org.jenkinsci.plugins.configfiles.'
+                         'buildwrapper.ConfigFileBuildWrapper')
+    xml_files = XML.SubElement(top, 'managedFiles')
+
+    files = data.get('files', [])
+    for file in files:
+        xml_file = XML.SubElement(xml_files, 'org.jenkinsci.plugins.'
+                                  'configfiles.buildwrapper.ManagedFile')
+        file_id = file.get('file-id')
+        if file_id is None:
+            raise JenkinsJobsException("file-id is required for each "
+                                       "managed configuration file")
+        XML.SubElement(xml_file, 'fileId').text = str(file_id)
+        XML.SubElement(xml_file, 'targetLocation').text = \
+            file.get('target', '')
+        XML.SubElement(xml_file, 'variable').text = \
+            file.get('variable', '')
+
+
 def logfilesize(parser, xml_parent, data):
     """yaml: logfilesize
     Abort the build if its logfile becomes too big.
@@ -694,27 +736,17 @@ def release(parser, xml_parent, data):
 
     :arg bool keep-forever: Keep build forever (default true)
     :arg bool override-build-parameters: Enable build-parameter override
-    :arg string version-template: Release version template
+        (default false)
+    :arg string version-template: Release version template (default '')
     :arg list parameters: Release parameters (see the :ref:`Parameters` module)
     :arg list pre-build: Pre-build steps (see the :ref:`Builders` module)
     :arg list post-build: Post-build steps (see :ref:`Builders`)
     :arg list post-success: Post successful-build steps (see :ref:`Builders`)
     :arg list post-failed: Post failed-build steps (see :ref:`Builders`)
 
-    Example::
+    Example:
 
-      wrappers:
-        - release:
-            keep-forever: false
-            parameters:
-                - string:
-                    name: RELEASE_BRANCH
-                    default: ''
-                    description: Git branch to release from.
-            post-success:
-                - shell: |
-                    #!/bin/bash
-                    copy_build_artefacts.sh
+    .. literalinclude:: /../../tests/wrappers/fixtures/release001.yaml
 
     """
     relwrap = XML.SubElement(xml_parent,
@@ -729,11 +761,11 @@ def release(parser, xml_parent, data):
         data.get('override-build-parameters', False)).lower()
     XML.SubElement(relwrap, 'releaseVersionTemplate').text = data.get(
         'version-template', '')
-    for param in data.get('parameters', []):
-        parser.registry.dispatch('parameter', parser,
-                                 XML.SubElement(relwrap,
-                                                'parameterDefinitions'),
-                                 param)
+    parameters = data.get('parameters', [])
+    if parameters:
+        pdef = XML.SubElement(relwrap, 'parameterDefinitions')
+        for param in parameters:
+            parser.registry.dispatch('parameter', parser, pdef, param)
 
     builder_steps = {
         'pre-build': 'preBuildSteps',
@@ -1086,6 +1118,61 @@ def ssh_agent_credentials(parser, xml_parent, data):
         raise JenkinsJobsException("Missing 'user' for ssh-agent-credentials")
 
 
+def credentials_binding(parser, xml_parent, data):
+    """yaml: credentials-binding
+    Binds credentials to environment variables using the credentials binding
+    plugin for jenkins.
+
+    Requires the Jenkins `Credentials Binding Plugin
+    <https://wiki.jenkins-ci.org/display/JENKINS/Credentials+Binding+Plugin>`_
+    version 1.1 or greater.
+
+    :arg list binding-type: List of each bindings to create.  Bindings may be\
+                            of type `zip-file`, `file`, `username-password`,\
+                            or `text`
+
+        :Parameters: * **credential-id** (`str`) UUID of the credential being\
+                                                 referenced
+                     * **variable** (`str`) Environment variable where the\
+                                            credential will be stored
+
+    Example:
+
+    .. literalinclude::
+            /../../tests/wrappers/fixtures/credentials_binding.yaml
+            :language: yaml
+
+    """
+    entry_xml = XML.SubElement(
+        xml_parent,
+        'org.jenkinsci.plugins.credentialsbinding.impl.SecretBuildWrapper')
+    bindings_xml = XML.SubElement(entry_xml, 'bindings')
+    binding_types = {
+        'zip-file': 'org.jenkinsci.plugins.credentialsbinding.impl.'
+                    'ZipFileBinding',
+        'file': 'org.jenkinsci.plugins.credentialsbinding.impl.FileBinding',
+        'username-password': 'org.jenkinsci.plugins.credentialsbinding.impl.'
+                             'UsernamePasswordBinding',
+        'text': 'org.jenkinsci.plugins.credentialsbinding.impl.StringBinding'
+    }
+    if not data:
+        raise JenkinsJobsException('At least one binding-type must be '
+                                   'specified for the credentials-binding '
+                                   'element')
+    for binding in data:
+        for binding_type, params in binding.items():
+            if binding_type not in binding_types.keys():
+                raise JenkinsJobsException('binding-type must be one of %r' %
+                                           binding_types.keys())
+
+            binding_xml = XML.SubElement(bindings_xml,
+                                         binding_types[binding_type])
+            variable_xml = XML.SubElement(binding_xml, 'variable')
+            variable_xml.text = params.get('variable')
+            credential_xml = XML.SubElement(binding_xml, 'credentialsId')
+            credential_xml.text = params.get('credential-id')
+
+
 class Wrappers(jenkins_jobs.modules.base.Base):
     sequence = 80
 
